@@ -45,6 +45,7 @@ xorg_backtrace(void)
 {
     unw_cursor_t cursor;
     unw_context_t context;
+    unw_word_t ip;
     unw_word_t off;
     unw_proc_info_t pip;
     int ret, i = 0;
@@ -88,7 +89,9 @@ xorg_backtrace(void)
             procname[1] = 0;
         }
 
-        if (dladdr((void *)(uintptr_t)(pip.start_ip + off), &dlinfo) && dlinfo.dli_fname &&
+        if (unw_get_reg (&cursor, UNW_REG_IP, &ip) < 0)
+          ip = pip.start_ip + off;
+        if (dladdr((void *)(uintptr_t)(ip), &dlinfo) && dlinfo.dli_fname &&
                 *dlinfo.dli_fname)
             filename = dlinfo.dli_fname;
         else
@@ -96,7 +99,7 @@ xorg_backtrace(void)
 
         ErrorFSigSafe("%u: %s (%s%s+0x%x) [%p]\n", i++, filename, procname,
             ret == -UNW_ENOMEM ? "..." : "", (int)off,
-            (void *)(uintptr_t)(pip.start_ip + off));
+            (void *)(uintptr_t)(ip));
 
         ret = unw_step(&cursor);
         if (ret < 0)
@@ -173,7 +176,7 @@ xorg_backtrace(void)
 #define ElfSym Elf32_Sym
 #endif
 
-/* Called for each frame on the stack to print it's contents */
+/* Called for each frame on the stack to print its contents */
 static int
 xorg_backtrace_frame(uintptr_t pc, int signo, void *arg)
 {
@@ -370,6 +373,11 @@ xorg_backtrace(void)
         return;
 }
 
+#endif
+#endif
+#endif
+
+#ifdef __CYGWIN__
 /* Cygwin-specific crash-reporter glue */
 
 #include <X11/Xwindows.h>
@@ -448,5 +456,3 @@ xorg_crashreport(int signo, siginfo_t *sip, void *sigcontext)
 }
 
 #endif
-#endif
-#endif
